package com.swagger.rest.controllers

import com.swagger.rest.models.Platform
import com.swagger.rest.models.PlatformInput
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class PlatformController(
    private val platformRepository: PlatformRepository,
    private val projectRepository: ProjectRepository
) {

    @GetMapping("/platforms")
    fun getPlatformByProjectId(@RequestParam(required = false) project: String?): ResponseEntity<List<Platform>?> {
        return try {
            val platforms: List<Platform> =
                if (project == null) {
                    platformRepository.findAll() as List<Platform>
                } else {
                    platformRepository.findByProject(project) as List<Platform>
                }
            if (platforms.isEmpty()) {
                ResponseEntity<List<Platform>?>(HttpStatus.NO_CONTENT)
            } else ResponseEntity<List<Platform>?>(platforms, HttpStatus.OK)
        } catch (e: Exception) {
            ResponseEntity<List<Platform>?>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/platforms/{id}")
    fun getPlatformById(@PathVariable("id") id: Long): ResponseEntity<Platform> {
        val platformData: Optional<Platform?> = platformRepository.findById(id)
        return if (platformData.isPresent) {
            ResponseEntity<Platform>(platformData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
        }
    }

    @PostMapping("/platforms")
    fun addPlatformByProjectId(
        @RequestBody platform: Platform, @RequestParam project: String
    ): ResponseEntity<Platform> {
        return try {
            val found = platformRepository.findByName(platform.name, project)!!.size
            val foundProject = projectRepository.findById(project.toLong()).isPresent
            if (platform.name.isNotBlank()) {
                if (!foundProject) {
                    ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
                } else if (platform.name.length > 100) {
                    ResponseEntity<Platform>(HttpStatus.PAYLOAD_TOO_LARGE)
                } else if (found > 0) {
                    ResponseEntity<Platform>(HttpStatus.CONFLICT)
                } else {
                    platform.project_id = projectRepository.findById(project.toLong()).get()
                    val savePlatform: Platform = platformRepository.save(platform)
                    ResponseEntity<Platform>(savePlatform, HttpStatus.CREATED)
                }
            } else {
                ResponseEntity<Platform>(HttpStatus.BAD_REQUEST)
            }
        } catch (e: Exception) {
            ResponseEntity<Platform>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/platforms/{id}")
    fun updatePlatformById(@PathVariable("id") id: Long, @RequestBody platform: PlatformInput): Any {
        val platformData: Optional<Platform?> = platformRepository.findById(id)
        val found = platformRepository.findByName(platform.name, platform.project_id.toString())!!.size
        val foundProject = projectRepository.findById(platform.project_id!!.toLong())
        return if (platform.name!!.isNotBlank()) {
            if (platform.name!!.length > 100) {
                ResponseEntity<Platform>(HttpStatus.PAYLOAD_TOO_LARGE)
            } else if (foundProject.isEmpty) {
                println(1)
                ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
            } else if (platformData.isPresent) {
                println(2)
                if (((platform.name == platformData.get().name && found > 0) || (platform.name !== platformData.get().name && found == 0))) {
                    println(3)
                    val savePlatform: Platform = platformData.get()
                    savePlatform.project_id = foundProject.get()
                    savePlatform.name = platform.name!!
                    ResponseEntity<Platform>(platformRepository.save(savePlatform), HttpStatus.OK)
                } else {
                    println(4)
                    ResponseEntity<Platform>(HttpStatus.CONFLICT)
                }
            } else {
                println(5)
                ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
            }
        } else {
            println(6)
            ResponseEntity<Platform>(HttpStatus.BAD_REQUEST)
        }
    }

    @DeleteMapping("/platforms/{id}")
    fun deletePlatformById(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val find = platformRepository.findById(id)
        return try {
            if (find.isPresent) {
                platformRepository.deleteById(id)
                ResponseEntity(HttpStatus.OK)
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }
}
