package com.swagger.rest.controllers

import com.swagger.rest.models.Project
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class ProjectController(
    private val projectRepository: ProjectRepository,
    private val platformRepository: PlatformRepository
) {

    @GetMapping("/projects")
    fun getProject(@RequestParam(required = false) name: String?): ResponseEntity<List<Project>?> {
        return try {
            val projects: List<Project> =
                if (name == null) projectRepository.findAll() as List<Project> else projectRepository.findByNameContaining(
                    name
                ) as List<Project>
            if (projects.isEmpty()) {
                ResponseEntity<List<Project>?>(HttpStatus.NO_CONTENT)
            } else ResponseEntity<List<Project>?>(projects, HttpStatus.OK)
        } catch (e: Exception) {
            ResponseEntity<List<Project>?>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/projects/{id}")
    fun getProjectById(@PathVariable("id") id: Long): ResponseEntity<Project> {
        val projectData: Optional<Project?> = projectRepository.findById(id)
        return if (projectData.isPresent) {
            ResponseEntity<Project>(projectData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<Project>(HttpStatus.NOT_FOUND)
        }
    }

    @PostMapping("/projects")
    fun addProject(@RequestBody project: Project): ResponseEntity<Project> {
        return try {
            val found = projectRepository.findByNameContaining(project.name)!!.size
            if (project.name.isNotBlank()) {
                if (project.name.length > 100 || project.description!!.length > 255) {
                    ResponseEntity<Project>(HttpStatus.PAYLOAD_TOO_LARGE)
                } else if (found > 0) {
                    ResponseEntity<Project>(HttpStatus.CONFLICT)
                } else {
                    val saveProject: Project = projectRepository.save(project)
                    ResponseEntity<Project>(saveProject, HttpStatus.CREATED)
                }
            } else {
                ResponseEntity<Project>(HttpStatus.BAD_REQUEST)
            }
        } catch (e: Exception) {
            ResponseEntity<Project>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/projects/{id}")
    fun updateProjectById(@PathVariable("id") id: Long, @RequestBody project: Project): ResponseEntity<out Any?> {
        val projectData: Optional<Project?> = projectRepository.findById(id)
        val found = projectRepository.findByNameContaining(project.name)!!.size
        return if (project.name.isNotBlank()) {
            if (project.name.length > 100 || project.description!!.length > 255) {
                ResponseEntity<Project>(HttpStatus.PAYLOAD_TOO_LARGE)
            } else if (projectData.isPresent) {
                if ((project.name == projectData.get().name && found > 0) || (project.name !== projectData.get().name && found == 0)) {
                    val saveProject: Project = projectData.get()
                    saveProject.name = project.name
                    saveProject.description = project.description
                    ResponseEntity<Any?>(projectRepository.save(saveProject), HttpStatus.OK)
                } else {
                    ResponseEntity<Project>(HttpStatus.CONFLICT)
                }
            } else {
                ResponseEntity<Project?>(HttpStatus.NOT_FOUND)
            }
        } else {
            ResponseEntity<Project?>(HttpStatus.BAD_REQUEST)
        }
    }

    @DeleteMapping("/projects/{id}")
    fun deleteProject(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val find = projectRepository.findById(id)
        val used = platformRepository.findByProject(id.toString())!!.size
        return try {
            if (used > 0) {
                ResponseEntity(HttpStatus.RESET_CONTENT)
            } else if (find.isPresent) {
                projectRepository.deleteById(id)
                ResponseEntity(HttpStatus.OK)
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }
}